#include <iostream>

using namespace std;
const int GRIDSIZE=100;

class Shape {
public:
  virtual bool inside(double x,double y) {}
  virtual void get_box(double box[4]) {}
};

double square(double x) { return x*x; }

class Circle : public Shape {
private:
  double xc,yc,R;
public:
  Circle(double xc_a,double yc_a,double R_a) 
    : xc(xc_a), yc(yc_a), R(R_a) {}
  bool inside(double x,double y) {
    return (square(x-xc)+square(y-yc))<square(R);
  }
  void get_box(double box[4]) {
    box[0] = xc-R;
    box[1] = xc+R;
    box[2] = yc-R;
    box[3] = yc+R;
  }
};

class Triangle : public Shape {
private:
  double xc,yc,Lx,Ly;
public:
  Triangle(double xc_a,double yc_a,double Lx_a,double Ly_a) 
    : xc(xc_a), yc(yc_a), Lx(Lx_a), Ly(Ly_a) {}
  bool inside(double x,double y) {
    return x-xc<=Lx && y-yc<=Ly && ((x-xc)/Lx + (y-yc)/Ly)<1;
  }
  void get_box(double box[4]) {
    box[0] = xc;
    box[1] = xc+Lx;
    box[2] = yc;
    box[3] = yc+Ly;
  }
};

bool overlap1d(double x1[2], double x2[2]) {
  if (x1[1]<x2[0]) return false;
  if (x2[1]<x1[0]) return false;
  return true;
}

double min(double x, double y) {
  return (x<=y? x : y);
}

double max(double x, double y) {
  return (x>=y? x : y);
}

bool overlap(Shape &s1,Shape &s2) {
  double box1[4],box2[4],box[4];
  s1.get_box(box1);
  s2.get_box(box2);
  double x1[2], x2[2];

  // Check overlap on x axis
  x1[0] = box1[0];
  x1[1] = box1[1];

  x2[0] = box2[0];
  x2[1] = box2[1];
  if (!overlap1d(x1,x2)) return false;

  // Check overlap on y axis
  x1[0] = box1[2];
  x1[1] = box1[3];

  x2[0] = box2[2];
  x2[1] = box2[3];
  if (!overlap1d(x1,x2)) return false;

  box[0] = min(box1[0],box2[0]);
  box[1] = max(box1[1],box2[1]);

  box[2] = min(box1[2],box2[2]);
  box[3] = max(box1[3],box2[3]);

  for (int j=0; j<=GRIDSIZE; j++) {
    double x = box[0] + j*((box[1]-box[0])/GRIDSIZE);
    for (int k=0; k<=GRIDSIZE; k++) {
      double y = box[2] + k*((box[3]-box[2])/GRIDSIZE);
      if (s1.inside(x,y) && s2.inside(x,y)) return true;
    }
  }  
  return false;
}

int main() {
  Circle c1(0,0,1),c2(1.4,1.4,1);
  Triangle t(-1.1,-1.1,1,1);

  cout << "overlap? " << overlap(c1,t) << endl;
  return 0;
}
