#include <cmath>
#include <iostream>
#include <vector>
using namespace std;

class vector_t {
public:
  virtual int size()=0;
  virtual double operator[](int j)=0;
};

double vecsum(vector_t &v) {
  int N = v.size();
  double sum = 0.0;
  for (int j=0; j<N; j++)  sum += v[j];
  return sum;
}

// A stride, linearly spaced values
class stride_t : public vector_t {
private:
  int N;
  double start, inc;
public:
  stride_t(int Na, double s,double i) :
    N(Na), start(s), inc(i) { }
  int size() { return N; }
  double operator[](int j) { return start+j*inc; }
};

// A wrapper to a vector<double>
class vecwrap_t : public vector_t {
private:
  const vector<double> *ap;
public:
  vecwrap_t(const vector<double> &a) : ap(&a) { }
  int size() { return ap->size(); }
  double operator[](int j) { return (*ap)[j]; }
};

// A plain function that computes its values
class fun_t : public vector_t {
public:
  int N;
  double p;
  fun_t() : N(10), p(3.0) { }
  int size() { return N; }
  double operator[](int j) { return pow(j,p); }
};

int main() {
  stride_t stride(10,0,1);
  cout << "sum(0..9): " << vecsum(stride) << endl;

  vector<double> a;
  double p=3; int N=10;
  a.resize(N);
  for (int j=0; j<N; j++) a[j] = pow(j,p);
  vecwrap_t vw(a); 
  cout << "sum_{j=0}^{j=9} x^3 (with vecwrap): "
       << vecsum(vw) << endl;

  fun_t f;
  cout << "sum_{j=0}^{j=9} x^3 (with fun_t): "
       << vecsum(f) << endl;

  return 0;
}
